//-----------------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from Arm Limited or its affiliates.
//
//            (C) COPYRIGHT 2013 Arm Limited or its affiliates.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from Arm Limited or its affiliates.
//
//      SVN Information
//
//      Checked In          : $Date: 2017-10-10 15:55:38 +0100 (Tue, 10 Oct 2017) $
//
//      Revision            : $Revision: 371321 $
//
//      Release Information : Cortex-M System Design Kit-r1p1-00rel0
//
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//     IOP interconnect for CORTEX-M0+
//-----------------------------------------------------------------------------
//
// This block contains IOP match logic and IOP read data multiplexors. It has
// 1 slave port and 3 master ports:
// Slave port    -   IOP from CORTEX-M0+ MTB Integration
// Master 0 port -   GPIO 0
// Master 1 port -   GPIO 1
//-----------------------------------------------------------------------------

module cmsdk_iop_interconnect
 #(// Peripheral is mapped to base region 0x40010000 by default
   parameter BASEADDR_GPIO0 = 32'h4001_0000,
   // Peripheral is mapped to base region 0x40011000 by default
   parameter BASEADDR_GPIO1 = 32'h4001_1000)
  (// Inputs
   input  wire [31:0] IOADDR,
   input  wire [31:0] IOCHECK,    // I/O Address query
   input  wire [31:0] IORDATA0,
   input  wire [31:0] IORDATA1,
   // Output wires
   output wire        IOSEL0,
   output wire        IOSEL1,
   output wire        IOMATCH,
   output wire [31:0] IORDATA
   );

//-----------------------------------------------------------------------------
// I/O Port Peripheral Target Hit
//-----------------------------------------------------------------------------

  wire  IOMATCH0 = (IOCHECK[31:12] == BASEADDR_GPIO0[31:12]) ?
        1'b1 : 1'b0;

  wire  IOMATCH1 = (IOCHECK[31:12] == BASEADDR_GPIO1[31:12]) ?
         1'b1 : 1'b0;

//-----------------------------------------------------------------------------
// IO Port register address control
//-----------------------------------------------------------------------------

   // GPIO 0 select : 0x4001_0000 to 0x4001_07FF
   assign IOSEL0    = (IOADDR[31:12] == BASEADDR_GPIO0[31:12]);

   // GPIO 1 select : 0x4001_1000 to 0x4001_17FF
   assign IOSEL1    = (IOADDR[31:12] == BASEADDR_GPIO1[31:12]);

   assign IOMATCH  =  IOMATCH0 | IOMATCH1;
   assign IORDATA  =  ({32{IOSEL0}} & IORDATA0) |
                      ({32{IOSEL1}} & IORDATA1);

endmodule
